@@ -242,7 +242,19 @@ static Bool RADEONGetLegacyConnectorInfoFromBIOS (ScrnInfoPtr pScrn)
 	    info->BiosConnector[i].ConnectorType = (tmp >> 12) & 0xf;
 	    info->BiosConnector[i].DDCType = (tmp >> 8) & 0xf;
 	    info->BiosConnector[i].DACType = tmp & 0x1;
-	    info->BiosConnector[i].TMDSType = tmp & 0x10;
+	    info->BiosConnector[i].TMDSType = (tmp >> 4) & 0x1;
+
+	    /* most XPRESS chips seem to specify DDC_CRT2 for their 
+	     * VGA DDC port, however DDC never seems to work on that
+	     * port.  Some have reported success on DDC_MONID, so 
+	     * lets see what happens with that.
+	     */
+	    if (info->ChipFamily == CHIP_FAMILY_RS400 &&
+		info->BiosConnector[i].ConnectorType == CONNECTOR_CRT &&
+		info->BiosConnector[i].DDCType == DDC_CRT2) {
+		info->BiosConnector[i].DDCType = DDC_MONID;
+	    }
+
 
 	}
     } else {
@@ -619,8 +631,7 @@ Bool RADEONGetTVInfoFromBIOS (xf86OutputPtr output) {
 		ErrorF("\n");
 
 		return TRUE;
-	    } else
-		return FALSE;
+	    }
 	}
     }
     return FALSE;
@@ -654,8 +665,11 @@ Bool RADEONGetClockInfoFromBIOS (ScrnInfoPtr pScrn)
 	    if (info->sclk == 0) info->sclk = 200;
 	    if (info->mclk == 0) info->mclk = 200;
 		
-	    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "ref_freq: %d, min_pll: %ld, max_pll: %ld, xclk: %d, sclk: %f, mclk: %f\n",
-		       pll->reference_freq, pll->min_pll_freq, pll->max_pll_freq, pll->xclk, info->sclk, info->mclk);
+	    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "ref_freq: %d, min_pll: %u, "
+		       "max_pll: %u, xclk: %d, sclk: %f, mclk: %f\n",
+		       pll->reference_freq, (unsigned)pll->min_pll_freq,
+		       (unsigned)pll->max_pll_freq, pll->xclk, info->sclk,
+		       info->mclk);
 
 	} else {
 	    pll_info_block = RADEON_BIOS16 (info->ROMHeaderStart + 0x30);
@@ -840,8 +854,9 @@ Bool RADEONGetTMDSInfoFromBIOS (xf86OutputPtr output)
 					   ((RADEON_BIOS8(tmp+i*6+9) & 0xf)<<12) |
 					   ((RADEON_BIOS8(tmp+i*6+11) & 0xf)<<16));
 		xf86DrvMsg(pScrn->scrnIndex, X_INFO, 
-			   "TMDS PLL from BIOS: %ld %lx\n", 
-			   radeon_output->tmds_pll[i].freq, radeon_output->tmds_pll[i].value);
+			   "TMDS PLL from BIOS: %u %x\n", 
+			   (unsigned)radeon_output->tmds_pll[i].freq,
+			   (unsigned)radeon_output->tmds_pll[i].value);
 		       
 		if (maxfreq == radeon_output->tmds_pll[i].freq) {
 		    radeon_output->tmds_pll[i].freq = 0xffffffff;
@@ -876,23 +891,6 @@ Bool RADEONGetTMDSInfoFromBIOS (xf86OutputPtr output)
 		}
 		return TRUE;
 	    }
-
-	    /* revision 4 has some problem as it appears in RV280, 
-	       comment it off for now, use default instead */ 
-	    /*    
-		  else if (RADEON_BIOS8(tmp) == 4) {
-		  int stride = 0;
-		  n = RADEON_BIOS8(tmp + 5) + 1;
-		  if (n > 4) n = 4;
-		  for (i=0; i<n; i++) {
-		  radeon_output->tmds_pll[i].value = RADEON_BIOS32(tmp+stride+0x08);
-		  radeon_output->tmds_pll[i].freq = RADEON_BIOS16(tmp+stride+0x10);
-		  if (i == 0) stride += 10;
-		  else stride += 6;
-		  }
-		  return TRUE;
-		  }
-	    */  
 	}
     }
     return FALSE;
@@ -1077,7 +1075,7 @@ RADEONRestoreBIOSRegBlock(ScrnInfoPtr pScrn, CARD16 table_offset)
 	case RADEON_TABLE_FLAG_WRITE_INDEXED:
 	    val = RADEON_BIOS32(offset);
 	    ErrorF("WRITE INDEXED: 0x%x 0x%x\n",
-		   index, val);
+		   index, (unsigned)val);
 	    OUTREG(RADEON_MM_INDEX, index);
 	    OUTREG(RADEON_MM_DATA, val);
 	    offset += 4;
@@ -1085,7 +1083,7 @@ RADEONRestoreBIOSRegBlock(ScrnInfoPtr pScrn, CARD16 table_offset)
 
 	case RADEON_TABLE_FLAG_WRITE_DIRECT:
 	    val = RADEON_BIOS32(offset);
-	    ErrorF("WRITE DIRECT: 0x%x 0x%x\n", index, val);
+	    ErrorF("WRITE DIRECT: 0x%x 0x%x\n", index, (unsigned)val);
 	    OUTREG(index, val);
 	    offset += 4;
 	    break;
@@ -1096,7 +1094,7 @@ RADEONRestoreBIOSRegBlock(ScrnInfoPtr pScrn, CARD16 table_offset)
 	    ormask = RADEON_BIOS32(offset);
 	    offset += 4;
 	    ErrorF("MASK INDEXED: 0x%x 0x%x 0x%x\n",
-		   index, andmask, ormask);
+		   index, (unsigned)andmask, (unsigned)ormask);
 	    OUTREG(RADEON_MM_INDEX, index);
 	    val = INREG(RADEON_MM_DATA);
 	    val = (val & andmask) | ormask;
@@ -1109,7 +1107,7 @@ RADEONRestoreBIOSRegBlock(ScrnInfoPtr pScrn, CARD16 table_offset)
 	    ormask = RADEON_BIOS32(offset);
 	    offset += 4;
 	    ErrorF("MASK DIRECT: 0x%x 0x%x 0x%x\n",
-		   index, andmask, ormask);
+		   index, (unsigned)andmask, (unsigned)ormask);
 	    val = INREG(index);
 	    val = (val & andmask) | ormask;
 	    OUTREG(index, val);
@@ -1198,7 +1196,7 @@ RADEONRestoreBIOSMemBlock(ScrnInfoPtr pScrn, CARD16 table_offset)
 	    offset += 2;
 
 	    ErrorF("INDEX RADEON_MEM_SDRAM_MODE_REG %x %x\n",
-		   RADEON_SDRAM_MODE_MASK, ormask);
+		   RADEON_SDRAM_MODE_MASK, (unsigned)ormask);
 
 	    /* can this use direct access? */
 	    OUTREG(RADEON_MM_INDEX, RADEON_MEM_SDRAM_MODE_REG);
@@ -1209,7 +1207,7 @@ RADEONRestoreBIOSMemBlock(ScrnInfoPtr pScrn, CARD16 table_offset)
 	    ormask = (CARD32)index << 24;
 
 	    ErrorF("INDEX RADEON_MEM_SDRAM_MODE_REG %x %x\n",
-		   RADEON_B3MEM_RESET_MASK, ormask);
+		   RADEON_B3MEM_RESET_MASK, (unsigned)ormask);
 
             /* can this use direct access? */
             OUTREG(RADEON_MM_INDEX, RADEON_MEM_SDRAM_MODE_REG);
@@ -1224,7 +1222,6 @@ static void
 RADEONRestoreBIOSPllBlock(ScrnInfoPtr pScrn, CARD16 table_offset)
 {
     RADEONInfoPtr info = RADEONPTR (pScrn);
-    unsigned char *RADEONMMIO = info->MMIO;
     CARD16 offset = table_offset;
     CARD8  index, shift;
     CARD32 andmask, ormask, val, clk_pwrmgt_cntl;
@@ -1298,7 +1295,7 @@ RADEONRestoreBIOSPllBlock(ScrnInfoPtr pScrn, CARD16 table_offset)
 	    offset++;
 
 	    ErrorF("PLL_MASK_BYTE 0x%x 0x%x 0x%x 0x%x\n", 
-		   index, shift, andmask, ormask);
+		   index, shift, (unsigned)andmask, (unsigned)ormask);
 	    val = INPLL(pScrn, index);
 	    val = (val & andmask) | ormask;
 	    OUTPLL(pScrn, index, val);
@@ -1306,7 +1303,7 @@ RADEONRestoreBIOSPllBlock(ScrnInfoPtr pScrn, CARD16 table_offset)
 
 	case RADEON_PLL_FLAG_WRITE:
 	    val = RADEON_BIOS32(offset);
-	    ErrorF("PLL_WRITE 0x%x 0x%x\n", index, val);
+	    ErrorF("PLL_WRITE 0x%x 0x%x\n", index, (unsigned)val);
 	    OUTPLL(pScrn, index, val);
 	    offset += 4;
 	    break;
